using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using Utils.DataBase;
using System.Data.OleDb;
using Utils.Logs;

namespace PhoenixDLL.Class
{
    /// <summary>
    /// Patient manager specialized in seeking methods
    /// </summary>
    public static class PatientSeeker
    {
        #region Methods
        /// <summary>
        /// Seek patients on profession
        /// </summary>
        /// <param name="_Profession">Patient's profession</param>
        /// <returns>DataSet with the information of the search</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="Exception">If un unattempted exception occures</exception>
        public static DataSet SeekByProfession(string _Profession)
        {
            try
            {
                OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
                DataSet ds = new DataSet();
                SqlManager.SqlQuery = "SELECT * FROM Patient WHERE Profession LIKE \"" + _Profession + "\";";
                SqlManager.ExecuteSql(ref ds);
                return ds;
            }
            catch (OleDbException ex)
            {
                Log.Instance.Write(ex, typeof(PatientSeeker), LogType.Error);
                throw;
            }
            catch (InvalidOperationException ex)
            {
                Log.Instance.Write(ex, typeof(PatientSeeker), LogType.Error);
                throw;
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex, typeof(PatientSeeker), LogType.Error);
                throw;
            }
        }
        /// <summary>
        ///  Seek patients on date of birth       
        /// </summary>
        /// <param name="_DateOfBirth">Patient's date of birth</param>
        /// <returns>DataSet with the information of the search</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="Exception">If un unattempted exception occures</exception>
        public static DataSet SeekByDateOfBirth(DateTime _DateOfBirth)
        {
            try
            {
                string dob = _DateOfBirth.Year + "/" + _DateOfBirth.Month.ToString("00") + "/" + _DateOfBirth.Day.ToString("00");
                OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
                DataSet ds = new DataSet();
                SqlManager.SqlQuery = "SELECT * FROM Patient WHERE DateNaissance = FORMAT(\"" + dob + "\",\"yyyy/mm/dd\");";
                SqlManager.ExecuteSql(ref ds);
                return ds;
            }
            catch (OleDbException ex)
            {
                Log.Instance.Write(ex, typeof(PatientSeeker), LogType.Error);
                throw;
            }
            catch (InvalidOperationException ex)
            {
                Log.Instance.Write(ex, typeof(PatientSeeker), LogType.Error);
                throw;
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex, typeof(PatientSeeker), LogType.Error);
                throw;
            }
        }
        /// <summary>
        /// Seek patient on last update
        /// </summary>
        /// <param name="_LastUpdate">Patient's last update date</param>
        /// <returns>DataSet with the information of the search</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="Exception">If un unattempted exception occures</exception>
        public static DataSet SeekByLastUpdate(DateTime _LastUpdate)
        {
            try
            {
                string dob = _LastUpdate.Year + "/" + _LastUpdate.Month.ToString("00") + "/" + _LastUpdate.Day.ToString("00");
                OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
                DataSet ds = new DataSet();
                SqlManager.SqlQuery = "SELECT * FROM Patient WHERE Derniere_MAJ = FORMAT(\"" + dob + "\",\"yyyy/mm/dd\");";
                SqlManager.ExecuteSql(ref ds);
                return ds;
            }
            catch (OleDbException ex)
            {
                Log.Instance.Write(ex, typeof(PatientSeeker), LogType.Error);
                throw;
            }
            catch (InvalidOperationException ex)
            {
                Log.Instance.Write(ex, typeof(PatientSeeker), LogType.Error);
                throw;
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex, typeof(PatientSeeker), LogType.Error);
                throw;
            }
        }
        /// <summary>
        /// Seek patients on zip code
        /// </summary>
        /// <param name="_ZipCode">Patient's zip code</param>        
        /// <returns>DataSet with the information of the search</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="Exception">If un unattempted exception occures</exception>
        public static DataSet SeekByZipCode(string _ZipCode)
        {
            try
            {
                OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
                DataSet ds = new DataSet();
                SqlManager.SqlQuery = "SELECT * FROM Patient WHERE Code_Postal LIKE \"" + _ZipCode + "\";";
                SqlManager.ExecuteSql(ref ds);
                return ds;
            }
            catch (OleDbException ex)
            {
                Log.Instance.Write(ex, typeof(PatientSeeker), LogType.Error);
                throw;
            }
            catch (InvalidOperationException ex)
            {
                Log.Instance.Write(ex, typeof(PatientSeeker), LogType.Error);
                throw;
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex, typeof(PatientSeeker), LogType.Error);
                throw;
            }
        }
        /// <summary>
        /// Seek patients on birth place
        /// </summary>
        /// <param name="_BirthPlace">Patient's Place of birth</param>        
        /// <returns>DataSet with the information of the search</returns>
        /// <exception cref="OleDbException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="Exception">If un unattempted exception occures</exception>
        public static DataSet SeekByPlaceOfBirth(string _BirthPlace)
        {
            try
            {
                OleDbSql SqlManager = new OleDbSql(Options.Instance.Path);
                DataSet ds = new DataSet();
                SqlManager.SqlQuery = "SELECT * FROM Patient WHERE Lieu_Naissance LIKE \"" + _BirthPlace + "\";";
                SqlManager.ExecuteSql(ref ds);
                return ds;
            }
            catch (OleDbException ex)
            {
                Log.Instance.Write(ex, typeof(PatientSeeker), LogType.Error);
                throw;
            }
            catch (InvalidOperationException ex)
            {
                Log.Instance.Write(ex, typeof(PatientSeeker), LogType.Error);
                throw;
            }
            catch (Exception ex)
            {
                Log.Instance.Write(ex, typeof(PatientSeeker), LogType.Error);
                throw;
            }
        }
        #endregion
    }
}
